# Generated by roxygen2: do not edit by hand

export("%>%")
export(clusterByDistance)
export(coreMetadataNames)
export(getAPIKey)
export(getLocationDataDir)
export(initializeMazamaSpatialUtils)
export(location_createID)
export(location_getCensusBlock)
export(location_getOpenCageInfo)
export(location_getSingleAddress_Photon)
export(location_getSingleAddress_TexasAM)
export(location_getSingleElevation_USGS)
export(location_initialize)
export(setAPIKey)
export(setLocationDataDir)
export(showAPIKeys)
export(table_addClustering)
export(table_addColumn)
export(table_addCoreMetadata)
export(table_addLocation)
export(table_addOpenCageInfo)
export(table_addSingleLocation)
export(table_filterByDistance)
export(table_findAdjacentDistances)
export(table_findAdjacentLocations)
export(table_getDistanceFromTarget)
export(table_getLocationID)
export(table_getNearestDistance)
export(table_getNearestLocation)
export(table_getRecordIndex)
export(table_initialize)
export(table_initializeExisting)
export(table_leaflet)
export(table_leafletAdd)
export(table_load)
export(table_removeColumn)
export(table_removeRecord)
export(table_save)
export(table_updateColumn)
export(table_updateSingleRecord)
export(validateLocationTbl)
export(validateLonLat)
export(validateLonsLats)
export(validateMazamaSpatialUtils)
importFrom(MazamaCoreUtils,getAPIKey)
importFrom(MazamaCoreUtils,setAPIKey)
importFrom(MazamaCoreUtils,showAPIKeys)
importFrom(MazamaCoreUtils,stopIfNull)
importFrom(MazamaCoreUtils,validateLonLat)
importFrom(MazamaCoreUtils,validateLonsLats)
importFrom(MazamaSpatialUtils,getCountryCode)
importFrom(MazamaSpatialUtils,getStateCode)
importFrom(MazamaSpatialUtils,getTimezone)
importFrom(dplyr,all_of)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,tibble)
importFrom(geodist,geodist)
importFrom(jsonlite,fromJSON)
importFrom(leaflet,addCircleMarkers)
importFrom(leaflet,addProviderTiles)
importFrom(leaflet,fitBounds)
importFrom(leaflet,leaflet)
importFrom(lubridate,now)
importFrom(magrittr,"%>%")
importFrom(methods,as)
importFrom(rlang,.data)
importFrom(stringr,str_detect)
importFrom(stringr,str_sub)
importFrom(stringr,str_subset)
importFrom(utils,capture.output)
