% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MazamaLocationUtils-package.R
\docType{data}
\name{coreMetadataNames}
\alias{coreMetadataNames}
\title{Names of standard spatial metadata columns}
\format{
A character vector
}
\usage{
coreMetadataNames
}
\description{
Character string identifiers of the minimum set of fields
required for a table to be considered a valid "known locations" table.

\preformatted{
coreMetadataNames <- c(
  "locationID",           # from MazamaLocationUtils::location_createID()
  "locationName",         # from MazamaLocationUtils::location_initialize()
  "longitude",            # user supplied
  "latitude",             # user supplied
  "elevation",            # from MazamaLocationUtils::getSingleElevation_USGS()
  "countryCode",          # from MazamaSpatialUtils::getCountryCode()
  "stateCode",            # from MazamaSpatialUtils::getStateCode()
  "countyName",           # from MazamaSpatialUtils::getUSCounty()
  "timezone",             # from MazamaSpatialUtils::getTimezone()
  "houseNumber",
  "street",
  "city",
  "postalCode"
)
}
}
\details{
coreMetadataNames
}
\keyword{datasets}
